// ReleaseCLI
// Copyright (C) 2023  Lilly Schramm <lilly@cdev.nexus>

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

// This package is responsible for delivering the cli.
package main

import (
	"log"
	"os"

	"github.com/urfave/cli/v2"
	"gitlab.eps-dev.de/Lilly/releasecli/src"
)

var copyRightText = `ReleaseCLI  Copyright (C) 2023  Lilly Schramm
This program comes with ABSOLUTELY NO WARRANTY;
This is free software, and you are welcome to redistribute it under certain conditions;
This program is licensed under GPL3;
See <https://gitlab.eps-dev.de/Lilly/releasecli/-/raw/main/LICENSE> for license text.`

func main() {
	releaseAssistant := src.NewReleaseAssistant()

	app := &cli.App{
		EnableBashCompletion: true,
		Version:              "v0.1.1",
		Authors: []*cli.Author{
			{
				Name:  "Lilly Schramm",
				Email: "lilly@cdev.nexus",
			},
		},
		Copyright:   copyRightText,
		Name:        "ReleaseCLI",
		Usage:       "Track changes without merge conflicts.",
		Description: "Track changes without merge conflicts.",
		Commands: []*cli.Command{
			{
				Name:    "change",
				Aliases: []string{"c"},
				Subcommands: []*cli.Command{
					{
						Name:    "create",
						Aliases: []string{"c"},
						Usage:   "create a new change",
						Action: func(_ *cli.Context) error {
							if err := releaseAssistant.RunCreateChange(); err != nil {
								log.Fatalln(err.Error())
							}

							return nil
						},
					},
				},
			},
			{
				Name:    "release",
				Aliases: []string{"r"},
				Subcommands: []*cli.Command{
					{
						Name:    "create",
						Aliases: []string{"c"},
						Usage:   "create a new release",
						Action: func(_ *cli.Context) error {
							if err := releaseAssistant.RunCreateRelease(); err != nil {
								log.Fatalln(err.Error())
							}

							return nil
						},
					},
				},
			},
			{
				Name:    "changelog",
				Aliases: []string{"cl"},
				Subcommands: []*cli.Command{
					{
						Name:    "generate",
						Aliases: []string{"g"},
						Usage:   "generate the changelog",
						Action: func(cCtx *cli.Context) error {
							stdout := cCtx.Bool("stdout")
							latest := cCtx.Bool("latest")

							if err := releaseAssistant.RunGenerateChangelog(stdout, latest); err != nil {
								log.Fatalln(err.Error())
							}

							return nil
						},
						Flags: []cli.Flag{
							&cli.BoolFlag{
								Name:  "stdout",
								Usage: "print the changelog to stdout instead of a file",
							},
							&cli.BoolFlag{
								Name:  "latest",
								Usage: "only generate the latest entry",
							},
						},
					},
				},
			},
			{
				Name:    "init",
				Aliases: []string{"i"},
				Usage:   "Initialize the project. This will create a config file in the current directory.",
				Action: func(_ *cli.Context) error {
					if err := releaseAssistant.RunInitializeProject(); err != nil {
						log.Fatalln(err.Error())
					}

					return nil
				},
			},
		},
	}

	if err := app.Run(os.Args); err != nil {
		log.Fatal(err)
	}
}
