// ReleaseCLI
// Copyright (C) 2023  Lilly Schramm <lilly@cdev.nexus>

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

package src

import (
	"log"
	"os"
)

func (r *ReleaseAssistant) ensureDirectoryExists(directory string) error {
	exists, err := pathExists(directory)
	if err != nil {
		return err
	}

	if exists {
		return nil
	}

	log.Printf("Directory '%s' not found. Creating...\n", directory)

	if err := os.MkdirAll(directory, os.ModePerm); err != nil {
		return err
	}

	log.Printf("Created directory '%s'.\n", directory)

	return nil
}

func pathExists(path string) (bool, error) {
	_, err := os.Stat(path)
	if err == nil {
		return true, nil
	}

	if os.IsNotExist(err) {
		return false, nil
	}

	return false, err
}
